@@ -1,29 +1,27 @@
 use std::collections::HashMap;
+use std::fs;
+use std::io::Read;
 use std::path::PathBuf;
-use std::{fs, io, result};
 
 use crate::global_fns::helpers::search_for_file;
 use config::Config;
-use libs::base64::encode as encode_b64;
+
+use libs::base64::engine::{general_purpose::STANDARD as standard_b64, Engine};
 use libs::sha2::{digest, Sha256, Sha384, Sha512};
 use libs::tera::{from_value, to_value, Function as TeraFn, Result, Value};
-use libs::url;
 use utils::site::resolve_internal_link;
 
-fn compute_file_hash<D: digest::Digest>(
-    mut file: fs::File,
-    as_base64: bool,
-) -> result::Result<String, io::Error>
+fn compute_hash<D: digest::Digest>(literal: String, as_base64: bool) -> String
 where
     digest::Output<D>: core::fmt::LowerHex,
     D: std::io::Write,
 {
     let mut hasher = D::new();
-    io::copy(&mut file, &mut hasher)?;
+    hasher.update(literal);
     if as_base64 {
-        Ok(encode_b64(hasher.finalize()))
+        standard_b64.encode(hasher.finalize())
     } else {
-        Ok(format!("{:x}", hasher.finalize()))
+        format!("{:x}", hasher.finalize())
     }
 }
 
@@ -104,10 +102,10 @@ impl TeraFn for GetUrl {
             // anything else
             let mut segments = vec![];
 
-            if lang != self.config.default_language {
-                if path.is_empty() || !path[1..].starts_with(&lang) {
-                    segments.push(lang);
-                }
+            if lang != self.config.default_language
+                && (path.is_empty() || !path[1..].starts_with(&lang))
+            {
+                segments.push(lang);
             }
 
             segments.push(path);
@@ -127,11 +125,17 @@ impl TeraFn for GetUrl {
                     &self.output_path,
                 )
                 .map_err(|e| format!("`get_url`: {}", e))?
-                .and_then(|(p, _)| fs::File::open(&p).ok())
-                .and_then(|f| compute_file_hash::<Sha256>(f, false).ok())
-                {
+                .and_then(|(p, _)| fs::File::open(p).ok())
+                .and_then(|mut f| {
+                    let mut contents = String::new();
+
+                    f.read_to_string(&mut contents).ok()?;
+
+                    Some(compute_hash::<Sha256>(contents, false))
+                }) {
                     Some(hash) => {
-                        permalink = format!("{}?h={}", permalink, hash);
+                        let shorthash = &hash[..20]; // 2^-80 chance of false positive
+                        permalink = format!("{}?h={}", permalink, shorthash);
                     }
                     None => {
                         return Err(format!(
@@ -143,19 +147,6 @@ impl TeraFn for GetUrl {
                 };
             }
 
-            if cfg!(target_os = "windows") {
-                permalink = match url::Url::parse(&permalink) {
-                    Ok(parsed) => parsed.into(),
-                    Err(_) => {
-                        return Err(format!(
-                            "`get_url`: Could not parse link `{}` as a valid URL",
-                            permalink
-                        )
-                        .into())
-                    }
-                };
-            }
-
             Ok(to_value(permalink).unwrap())
         }
     }
@@ -166,71 +157,97 @@ impl TeraFn for GetUrl {
 }
 
 #[derive(Debug)]
-pub struct GetFileHash {
+pub struct GetHash {
     base_path: PathBuf,
     theme: Option<String>,
     output_path: PathBuf,
 }
-impl GetFileHash {
+impl GetHash {
     pub fn new(base_path: PathBuf, theme: Option<String>, output_path: PathBuf) -> Self {
         Self { base_path, theme, output_path }
     }
 }
 
-impl TeraFn for GetFileHash {
+impl TeraFn for GetHash {
     fn call(&self, args: &HashMap<String, Value>) -> Result<Value> {
-        let path = required_arg!(
+        let path = optional_arg!(
             String,
             args.get("path"),
-            "`get_file_hash` requires a `path` argument with a string value"
+            "`get_hash` requires either a `path` or a `literal` argument with a string value"
         );
+
+        let literal = optional_arg!(
+            String,
+            args.get("literal"),
+            "`get_hash` requires either a `path` or a `literal` argument with a string value"
+        );
+
+        let contents = match (path, literal) {
+            (Some(_), Some(_)) => {
+                return Err("`get_hash`: must have only one of `path` or `literal` argument".into());
+            }
+            (None, None) => {
+                return Err(
+                    "`get_hash`: must have at least one of `path` or `literal` argument".into()
+                );
+            }
+            (Some(path_v), None) => {
+                let file_path =
+                    match search_for_file(&self.base_path, &path_v, &self.theme, &self.output_path)
+                        .map_err(|e| format!("`get_hash`: {}", e))?
+                    {
+                        Some((f, _)) => f,
+                        None => {
+                            return Err(format!("`get_hash`: Cannot find file: {}", path_v).into());
+                        }
+                    };
+
+                let mut f = match std::fs::File::open(file_path) {
+                    Ok(f) => f,
+                    Err(e) => {
+                        return Err(format!("File {} could not be open: {}", path_v, e).into());
+                    }
+                };
+
+                let mut contents = String::new();
+
+                match f.read_to_string(&mut contents) {
+                    Ok(f) => f,
+                    Err(e) => {
+                        return Err(format!("File {} could not be read: {}", path_v, e).into());
+                    }
+                };
+
+                contents
+            }
+            (None, Some(literal_v)) => literal_v,
+        };
+
         let sha_type = optional_arg!(
             u16,
             args.get("sha_type"),
-            "`get_file_hash`: `sha_type` must be 256, 384 or 512"
+            "`get_hash`: `sha_type` must be 256, 384 or 512"
         )
         .unwrap_or(384);
-        let base64 = optional_arg!(
-            bool,
-            args.get("base64"),
-            "`get_file_hash`: `base64` must be true or false"
-        )
-        .unwrap_or(true);
 
-        let file_path =
-            match search_for_file(&self.base_path, &path, &self.theme, &self.output_path)
-                .map_err(|e| format!("`get_file_hash`: {}", e))?
-            {
-                Some((f, _)) => f,
-                None => {
-                    return Err(format!("`get_file_hash`: Cannot find file: {}", path).into());
-                }
-            };
-
-        let f = match std::fs::File::open(file_path) {
-            Ok(f) => f,
-            Err(e) => {
-                return Err(format!("File {} could not be open: {}", path, e).into());
-            }
-        };
+        let base64 =
+            optional_arg!(bool, args.get("base64"), "`get_hash`: `base64` must be true or false")
+                .unwrap_or(true);
 
         let hash = match sha_type {
-            256 => compute_file_hash::<Sha256>(f, base64),
-            384 => compute_file_hash::<Sha384>(f, base64),
-            512 => compute_file_hash::<Sha512>(f, base64),
-            _ => return Err("`get_file_hash`: Invalid sha value".into()),
+            256 => compute_hash::<Sha256>(contents, base64),
+            384 => compute_hash::<Sha384>(contents, base64),
+            512 => compute_hash::<Sha512>(contents, base64),
+            _ => return Err("`get_hash`: Invalid sha value".into()),
         };
 
-        match hash {
-            Ok(digest) => Ok(to_value(digest).unwrap()),
-            Err(_) => Err("`get_file_hash`: could no compute hash".into()),
-        }
+        Ok(to_value(hash).unwrap())
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::{GetFileHash, GetUrl};
+    use super::{GetHash, GetUrl};
 
     use std::collections::HashMap;
     use std::fs::create_dir;
@@ -272,7 +289,10 @@ title = "A title"
         let mut args = HashMap::new();
         args.insert("path".to_string(), to_value("app.css").unwrap());
         args.insert("cachebust".to_string(), to_value(true).unwrap());
-        assert_eq!(static_fn.call(&args).unwrap(), "http://a-website.com/app.css?h=572e691dc68c3fcd653ae463261bdb38f35dc6f01715d9ce68799319dd158840");
+        assert_eq!(
+            static_fn.call(&args).unwrap(),
+            "http://a-website.com/app.css?h=572e691dc68c3fcd653a"
+        );
     }
 
     #[test]
@@ -303,7 +323,10 @@ title = "A title"
         args.insert("path".to_string(), to_value("app.css").unwrap());
         args.insert("trailing_slash".to_string(), to_value(true).unwrap());
         args.insert("cachebust".to_string(), to_value(true).unwrap());
-        assert_eq!(static_fn.call(&args).unwrap(), "http://a-website.com/app.css/?h=572e691dc68c3fcd653ae463261bdb38f35dc6f01715d9ce68799319dd158840");
+        assert_eq!(
+            static_fn.call(&args).unwrap(),
+            "http://a-website.com/app.css/?h=572e691dc68c3fcd653a"
+        );
     }
 
     #[test]
@@ -456,7 +479,7 @@ title = "A title"
     #[test]
     fn can_get_file_hash_sha256_no_base64() {
         let dir = create_temp_dir();
-        let static_fn = GetFileHash::new(dir.into_path(), None, PathBuf::new());
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
         let mut args = HashMap::new();
         args.insert("path".to_string(), to_value("app.css").unwrap());
         args.insert("sha_type".to_string(), to_value(256).unwrap());
@@ -470,7 +493,7 @@ title = "A title"
     #[test]
     fn can_get_file_hash_sha256_base64() {
         let dir = create_temp_dir();
-        let static_fn = GetFileHash::new(dir.into_path(), None, PathBuf::new());
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
         let mut args = HashMap::new();
         args.insert("path".to_string(), to_value("app.css").unwrap());
         args.insert("sha_type".to_string(), to_value(256).unwrap());
@@ -481,7 +504,7 @@ title = "A title"
     #[test]
     fn can_get_file_hash_sha384_no_base64() {
         let dir = create_temp_dir();
-        let static_fn = GetFileHash::new(dir.into_path(), None, PathBuf::new());
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
         let mut args = HashMap::new();
         args.insert("path".to_string(), to_value("app.css").unwrap());
         args.insert("base64".to_string(), to_value(false).unwrap());
@@ -494,7 +517,7 @@ title = "A title"
     #[test]
     fn can_get_file_hash_sha384() {
         let dir = create_temp_dir();
-        let static_fn = GetFileHash::new(dir.into_path(), None, PathBuf::new());
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
         let mut args = HashMap::new();
         args.insert("path".to_string(), to_value("app.css").unwrap());
         assert_eq!(
@@ -506,7 +529,7 @@ title = "A title"
     #[test]
     fn can_get_file_hash_sha512_no_base64() {
         let dir = create_temp_dir();
-        let static_fn = GetFileHash::new(dir.into_path(), None, PathBuf::new());
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
         let mut args = HashMap::new();
         args.insert("path".to_string(), to_value("app.css").unwrap());
         args.insert("sha_type".to_string(), to_value(512).unwrap());
@@ -520,7 +543,7 @@ title = "A title"
     #[test]
     fn can_get_file_hash_sha512() {
         let dir = create_temp_dir();
-        let static_fn = GetFileHash::new(dir.into_path(), None, PathBuf::new());
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
         let mut args = HashMap::new();
         args.insert("path".to_string(), to_value("app.css").unwrap());
         args.insert("sha_type".to_string(), to_value(512).unwrap());
@@ -531,30 +554,86 @@ title = "A title"
     }
 
     #[test]
-    fn can_resolve_asset_path_to_valid_url() {
-        let config = Config::parse(CONFIG_DATA).unwrap();
+    fn can_get_hash_sha256_no_base64() {
         let dir = create_temp_dir();
-        let static_fn =
-            GetUrl::new(dir.path().to_path_buf(), config, HashMap::new(), PathBuf::new());
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
         let mut args = HashMap::new();
-        args.insert(
-            "path".to_string(),
-            to_value(dir.path().join("app.css").strip_prefix(std::env::temp_dir()).unwrap())
-                .unwrap(),
-        );
+        args.insert("literal".to_string(), to_value("Hello World").unwrap());
+        args.insert("sha_type".to_string(), to_value(256).unwrap());
+        args.insert("base64".to_string(), to_value(false).unwrap());
         assert_eq!(
             static_fn.call(&args).unwrap(),
-            format!(
-                "https://remplace-par-ton-url.fr/{}/app.css",
-                dir.path().file_stem().unwrap().to_string_lossy()
-            )
-        )
+            "a591a6d40bf420404a011733cfb7b190d62c65bf0bcda32b57b277d9ad9f146e"
+        );
+    }
+
+    #[test]
+    fn can_get_hash_sha256_base64() {
+        let dir = create_temp_dir();
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
+        let mut args = HashMap::new();
+        args.insert("literal".to_string(), to_value("Hello World").unwrap());
+        args.insert("sha_type".to_string(), to_value(256).unwrap());
+        args.insert("base64".to_string(), to_value(true).unwrap());
+        assert_eq!(static_fn.call(&args).unwrap(), "pZGm1Av0IEBKARczz7exkNYsZb8LzaMrV7J32a2fFG4=");
+    }
+
+    #[test]
+    fn can_get_hash_sha384_no_base64() {
+        let dir = create_temp_dir();
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
+        let mut args = HashMap::new();
+        args.insert("literal".to_string(), to_value("Hello World").unwrap());
+        args.insert("base64".to_string(), to_value(false).unwrap());
+        assert_eq!(
+            static_fn.call(&args).unwrap(),
+            "99514329186b2f6ae4a1329e7ee6c610a729636335174ac6b740f9028396fcc803d0e93863a7c3d90f86beee782f4f3f"
+            );
+    }
+
+    #[test]
+    fn can_get_hash_sha384() {
+        let dir = create_temp_dir();
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
+        let mut args = HashMap::new();
+        args.insert("literal".to_string(), to_value("Hello World").unwrap());
+        assert_eq!(
+            static_fn.call(&args).unwrap(),
+            "mVFDKRhrL2rkoTKefubGEKcpY2M1F0rGt0D5AoOW/MgD0Ok4Y6fD2Q+Gvu54L08/"
+        );
+    }
+
+    #[test]
+    fn can_get_hash_sha512_no_base64() {
+        let dir = create_temp_dir();
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
+        let mut args = HashMap::new();
+        args.insert("literal".to_string(), to_value("Hello World").unwrap());
+        args.insert("sha_type".to_string(), to_value(512).unwrap());
+        args.insert("base64".to_string(), to_value(false).unwrap());
+        assert_eq!(
+            static_fn.call(&args).unwrap(),
+            "2c74fd17edafd80e8447b0d46741ee243b7eb74dd2149a0ab1b9246fb30382f27e853d8585719e0e67cbda0daa8f51671064615d645ae27acb15bfb1447f459b"
+        );
+    }
+
+    #[test]
+    fn can_get_hash_sha512() {
+        let dir = create_temp_dir();
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
+        let mut args = HashMap::new();
+        args.insert("literal".to_string(), to_value("Hello World").unwrap());
+        args.insert("sha_type".to_string(), to_value(512).unwrap());
+        assert_eq!(
+            static_fn.call(&args).unwrap(),
+            "LHT9F+2v2A6ER7DUZ0HuJDt+t03SFJoKsbkkb7MDgvJ+hT2FhXGeDmfL2g2qj1FnEGRhXWRa4nrLFb+xRH9Fmw=="
+        );
     }
 
     #[test]
     fn error_when_file_not_found_for_hash() {
         let dir = create_temp_dir();
-        let static_fn = GetFileHash::new(dir.into_path(), None, PathBuf::new());
+        let static_fn = GetHash::new(dir.into_path(), None, PathBuf::new());
         let mut args = HashMap::new();
         args.insert("path".to_string(), to_value("doesnt-exist").unwrap());
         let err = format!("{}", static_fn.call(&args).unwrap_err());
