@@ -21,7 +21,7 @@ pub fn is_path_in_directory(parent: &Path, path: &Path) -> Result<bool> {
 /// Create a file with the content given
 pub fn create_file(path: &Path, content: &str) -> Result<()> {
     let mut file =
-        File::create(&path).with_context(|| format!("Failed to create file {}", path.display()))?;
+        File::create(path).with_context(|| format!("Failed to create file {}", path.display()))?;
     file.write_all(content.as_bytes())?;
     Ok(())
 }
@@ -92,19 +92,19 @@ pub fn copy_file_if_needed(src: &Path, dest: &Path, hard_link: bool) -> Result<(
             .with_context(|| format!("Failed to get metadata of {}", src.display()))?;
         let src_mtime = FileTime::from_last_modification_time(&src_metadata);
         if Path::new(&dest).is_file() {
-            let target_metadata = metadata(&dest)?;
+            let target_metadata = metadata(dest)?;
             let target_mtime = FileTime::from_last_modification_time(&target_metadata);
             if !(src_mtime == target_mtime && src_metadata.len() == target_metadata.len()) {
-                copy(src, &dest).with_context(|| {
+                copy(src, dest).with_context(|| {
                     format!("Was not able to copy file {} to {}", src.display(), dest.display())
                 })?;
-                set_file_mtime(&dest, src_mtime)?;
+                set_file_mtime(dest, src_mtime)?;
             }
         } else {
-            copy(src, &dest).with_context(|| {
+            copy(src, dest).with_context(|| {
                 format!("Was not able to copy directory {} to {}", src.display(), dest.display())
             })?;
-            set_file_mtime(&dest, src_mtime)?;
+            set_file_mtime(dest, src_mtime)?;
         }
     }
     Ok(())
@@ -166,6 +166,41 @@ where
     time_source.and_then(|ts| time_target.map(|tt| ts > tt)).unwrap_or(true)
 }
 
+/// Checks if the file or folder for the given path is a dotfile, meaning starts with '.'
+pub fn is_dotfile<P>(path: P) -> bool
+where
+    P: AsRef<Path>,
+{
+    path.as_ref().file_name().and_then(|s| s.to_str()).map(|s| s.starts_with('.')).unwrap_or(false)
+}
+
+/// Returns whether the path we received corresponds to a temp file created
+/// by an editor or the OS
+pub fn is_temp_file(path: &Path) -> bool {
+    let ext = path.extension();
+    match ext {
+        Some(ex) => match ex.to_str().unwrap() {
+            "swp" | "swx" | "tmp" | ".DS_STORE" | ".DS_Store" => true,
+            // jetbrains IDE
+            x if x.ends_with("jb_old___") => true,
+            x if x.ends_with("jb_tmp___") => true,
+            x if x.ends_with("jb_bak___") => true,
+            // vim & jetbrains
+            x if x.ends_with('~') => true,
+            _ => {
+                if let Some(filename) = path.file_stem() {
+                    // emacs
+                    let name = filename.to_str().unwrap();
+                    name.starts_with('#') || name.starts_with(".#")
+                } else {
+                    false
+                }
+            }
+        },
+        None => true,
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use std::fs::{metadata, read_to_string, File};
