@@ -9,3 +9,8 @@ pub fn get_available_port(avoid: u16) -> Option<u16> {
 pub fn port_is_available(port: u16) -> bool {
     TcpListener::bind(("127.0.0.1", port)).is_ok()
 }
+
+/// Returns whether a link starts with an HTTP(s) scheme or "www.".
+pub fn is_external_link(link: &str) -> bool {
+    link.starts_with("http:") || link.starts_with("https:") || link.starts_with("www.")
+}
